// Copyright (c) 2023, Tencent Inc.
// All rights reserved.
#include <fcntl.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "list.h"
#include "wmpf/hardware/video.h"
#include "wmpf/voip.h"
#include "wmpf/wmpf.h"

static list_declare(video_stream_list);
extern wx_voip_session_status_t voip_status;

typedef struct {
  struct wx_video_stream_config config;
  const struct wx_video_stream_config* listener;
  struct wx_video_stream* stream;
  struct listnode slist;
} VIDEO_STREAM;

static int video_fd = 0;

int voip_receive_video = 0;

static wx_error_t video_stream_write(
    struct wx_video_stream* stream,
    const void* buffer,
    size_t buffer_size,
    const struct wx_video_stream_data_metadata* metadata) {
      
  printf("%s size:%zu\n", __FUNCTION__, buffer_size);

  voip_receive_video = 1;

  if (video_fd == 0) {
    video_fd = open("video.h264", O_RDWR | O_CREAT, 0700);
  }

  if (video_fd <= 0) {
    printf("open video.h264 fail\n");
    return WXERROR_RESOURCE_EXHAUSTED;
  }

  write(video_fd, buffer, buffer_size);

  return WXERROR_OK;
}

static void video_stream_close(struct wx_video_stream* stream) {
  printf("%s \n", __FUNCTION__);
}

static struct wx_video_stream* alloc_video_stream(void) {
  struct wx_video_stream* stream =
      (struct wx_video_stream*)calloc(1, sizeof(struct wx_video_stream));

  if (stream) {
    stream->common.size = sizeof(struct wx_video_stream);
    stream->common.tag = WX_VIDEO_STREAM_TAG;
    stream->common.version = 0;

    stream->write = video_stream_write;
    stream->close = video_stream_close;
  }

  return stream;
}

static wx_error_t video_create_output_stream(
    struct wx_video_module* module,
    const struct wx_video_stream_config* config,
    struct wx_video_stream** stream_out) {
  VIDEO_STREAM* stream = (VIDEO_STREAM*)calloc(1, sizeof(VIDEO_STREAM));

  printf("%s \n", __FUNCTION__);

  if (stream) {
    struct wx_video_stream* _stream = alloc_video_stream();

    if (_stream) {
      printf("video format = %x\n", config->format);

      memcpy(&stream->config, config, sizeof(struct wx_video_stream_config));
      stream->stream = _stream;

      list_add_tail(&video_stream_list, &stream->slist);
      *stream_out = _stream;

      return WXERROR_OK;
    }

    free(stream);
  }
}

struct wx_video_module video_module = {
    .common =
        {
            .common =
                {
                    .tag = WX_VIDEO_MODULE_TAG,
                    .size = sizeof(wx_video_module_t),
                    .version = 0,
                },
            .id = WX_VIDEO_MODULE_ID,
            .set_on_devices_changed = NULL,
        },

    .create_encoder = NULL,
    .create_decoder = NULL,
    .create_output_stream = video_create_output_stream,
};
